<?php
/*##################################################
 *                          BlogsDisplayBlogsTagController.class.php
 *                            -------------------
 *   begin                : February 26, 2013
 *   copyright            : (C) 2013 Kevin MASSY
 *   email                : kevin.massy@phpboost.com
 *
 *
 ###################################################
 *	Modules News : Dupliqu et modifi
 *--------------------------------------------------
 * Date: 01.06.2016
 * Par: Swan | EDN
 * Site: http://easy-design.net
 * Contact : contact@easy-design.net
 *
 ###################################################
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 ###################################################*/

/**
 * @author Kevin MASSY <kevin.massy@phpboost.com>
 */
class BlogsDisplayBlogsTagController extends ModuleController
{
	private $tpl;
	private $lang;
	
	/**
	 * @var BlogsKeyword
	 */
	private $keyword;
	
	public function execute(HTTPRequestCustom $request)
	{
		$this->check_authorizations();
		
		$this->init();
		
		$this->build_view();
		
		return $this->generate_response();
	}
	
	public function init()
	{
		$this->lang = LangLoader::get('common', 'blogs');
		$this->tpl = new FileTemplate('blogs/BlogsDisplaySeveralBlogsController.tpl');
		$this->tpl->add_lang($this->lang);
	}
	
	public function build_view()
	{
		$now = new Date();
		$authorized_categories = BlogsService::get_authorized_categories(Category::ROOT_CATEGORY);
		$blogs_config = BlogsConfig::load();
		
		$condition = 'WHERE relation.id_keyword = :id_keyword
		AND id_category IN :authorized_categories
		AND (approbation_type = 1 OR (approbation_type = 2 AND start_date < :timestamp_now AND (end_date > :timestamp_now OR end_date = 0)))';
		$parameters = array(
			'id_keyword' => $this->get_keyword()->get_id(),
			'authorized_categories' => $authorized_categories,
			'timestamp_now' => $now->get_timestamp()
		);
		
		$page = AppContext::get_request()->get_getint('page', 1);
		$pagination = $this->get_pagination($condition, $parameters, $page);
		
		$result = PersistenceContext::get_querier()->select('SELECT blogs.*, member.*
		FROM '. BlogsSetup::$blogs_table .' blogs
		LEFT JOIN '. DB_TABLE_KEYWORDS_RELATIONS .' relation ON relation.module_id = \'blogs\' AND relation.id_in_module = blogs.id 
		LEFT JOIN '. DB_TABLE_MEMBER .' member ON member.user_id = blogs.author_user_id
		' . $condition . '
		ORDER BY top_list_enabled DESC, blogs.creation_date DESC
		LIMIT :number_items_per_page OFFSET :display_from', array_merge($parameters, array(
			'number_items_per_page' => $pagination->get_number_items_per_page(),
			'display_from' => $pagination->get_display_from()
		)));
		
		$number_columns_display_blogs = $blogs_config->get_number_columns_display_blogs();
		$this->tpl->put_all(array(
			'C_DISPLAY_BLOCK_TYPE' => $blogs_config->get_display_type() == BlogsConfig::DISPLAY_BLOCK,
			'C_DISPLAY_LIST_TYPE' => $blogs_config->get_display_type() == BlogsConfig::DISPLAY_LIST,
			'C_DISPLAY_CONDENSED_CONTENT' => $blogs_config->get_display_condensed_enabled(),
			'C_COMMENTS_ENABLED' => $blogs_config->get_comments_enabled(),
			
			'C_BLOGS_NO_AVAILABLE' => $result->get_rows_count() == 0,
			'C_PAGINATION' => $pagination->has_several_pages(),
			
			'PAGINATION' => $pagination->display(),
			'C_SEVERAL_COLUMNS' => $number_columns_display_blogs > 1,
			'NUMBER_COLUMNS' => $number_columns_display_blogs,
			'CATEGORY_NAME' => $this->get_keyword()->get_name()
		));
		
		while ($row = $result->fetch())
		{
			$blogs = new Blogs();
			$blogs->set_properties($row);
			
			$this->tpl->assign_block_vars('blogs', array_merge($blogs->get_array_tpl_vars(), array(
				'L_COMMENTS' => CommentsService::get_number_and_lang_comments('blogs', $row['id']),
				'NUMBER_COM' => !empty($row['number_comments']) ? $row['number_comments'] : 0
			)));
			$this->build_sources_view($blogs);
		}
		$result->dispose();
	}
	
	private function build_sources_view(Blogs $blogs)
	{
		$sources = $blogs->get_sources();
		$nbr_sources = count($sources);
		if ($nbr_sources)
		{
			$this->tpl->put('blogs.C_SOURCES', $nbr_sources > 0);
			
			$i = 1;
			foreach ($sources as $name => $url)
			{	
				$this->tpl->assign_block_vars('blogs.sources', array(
					'C_SEPARATOR' => $i < $nbr_sources,
					'NAME' => $name,
					'URL' => $url,
				));
				$i++;
			}
		}
	}
	
	private function get_keyword()
	{
		if ($this->keyword === null)
		{
			$rewrited_name = AppContext::get_request()->get_getstring('tag', '');
			if (!empty($rewrited_name))
			{
				try {
					$this->keyword = BlogsService::get_keywords_manager()->get_keyword('WHERE rewrited_name=:rewrited_name', array('rewrited_name' => $rewrited_name));
				} catch (RowNotFoundException $e) {
					$error_controller = PHPBoostErrors::unexisting_page();
					DispatchManager::redirect($error_controller);
				}
			}
			else
			{
				$error_controller = PHPBoostErrors::unexisting_page();
				DispatchManager::redirect($error_controller);
			}
		}
		return $this->keyword;
	}
	
	private function get_pagination($condition, $parameters, $page)
	{
		$result = PersistenceContext::get_querier()->select_single_row_query('SELECT COUNT(*) AS nbr_blogs
		FROM '. BlogsSetup::$blogs_table .' blogs
		LEFT JOIN '. DB_TABLE_KEYWORDS_RELATIONS .' relation ON relation.module_id = \'blogs\' AND relation.id_in_module = blogs.id 
		' . $condition, $parameters);

		$pagination = new ModulePagination($page, $result['nbr_blogs'], (int)BlogsConfig::load()->get_number_blogs_per_page());
		$pagination->set_url(BlogsUrlBuilder::display_tag($this->get_keyword()->get_rewrited_name(), '%d'));
		
		if ($pagination->current_page_is_empty() && $page > 1)
        {
			$error_controller = PHPBoostErrors::unexisting_page();
			DispatchManager::redirect($error_controller);
		}
		
		return $pagination;
	}
	
	private function check_authorizations()
	{
		if (!BlogsAuthorizationsService::check_authorizations()->read())
		{
			$error_controller = PHPBoostErrors::user_not_authorized();
	   		DispatchManager::redirect($error_controller);
		}
	}
	
	private function generate_response()
	{
		$response = new SiteDisplayResponse($this->tpl);
		
		$graphical_environment = $response->get_graphical_environment();
		$graphical_environment->set_page_title($this->get_keyword()->get_name(), $this->lang['blogs']);
		$graphical_environment->get_seo_meta_data()->set_description(StringVars::replace_vars($this->lang['blogs.seo.description.tag'], array('subject' => $this->get_keyword()->get_name())));
		$graphical_environment->get_seo_meta_data()->set_canonical_url(BlogsUrlBuilder::display_tag($this->get_keyword()->get_rewrited_name(), AppContext::get_request()->get_getint('page', 1)));
		
		$breadcrumb = $graphical_environment->get_breadcrumb();
		$breadcrumb->add($this->lang['blogs'], BlogsUrlBuilder::home());
		$breadcrumb->add($this->get_keyword()->get_name(), BlogsUrlBuilder::display_tag($this->get_keyword()->get_rewrited_name(), AppContext::get_request()->get_getint('page', 1)));
		
		return $response;
	}
}
?>