<?php
/*##################################################
 *		                         BlogsFormController.class.php
 *                            -------------------
 *   begin                : February 13, 2013
 *   copyright            : (C) 2013 Kevin MASSY
 *   email                : kevin.massy@phpboost.com
 *
 *
 ###################################################
 *	Modules News : Dupliqu et modifi
 *--------------------------------------------------
 * Date: 01.06.2016
 * Par: Swan | EDN
 * Site: http://easy-design.net
 * Contact : contact@easy-design.net
 *
 ###################################################
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 ###################################################*/

class BlogsFormController extends ModuleController
{
	/**
	 * @var HTMLForm
	 */
	private $form;
	/**
	 * @var FormButtonSubmit
	 */
	private $submit_button;
	
	private $lang;
	private $common_lang;
	
	private $blogs;
	private $is_new_blogs;
	
	public function execute(HTTPRequestCustom $request)
	{
		$this->init();
		
		$this->check_authorizations();
		
		$this->build_form($request);
		
		$tpl = new StringTemplate('# INCLUDE FORM #');
		$tpl->add_lang($this->lang);
		
		if ($this->submit_button->has_been_submited() && $this->form->validate())
		{
			$this->save();
			$this->redirect();
		}
		
		$tpl->put('FORM', $this->form->display());
		
		return $this->generate_response($tpl);
	}
	
	private function init()
	{
		$this->lang = LangLoader::get('common', 'blogs');
		$this->common_lang = LangLoader::get('common');
	}
	
	private function build_form(HTTPRequestCustom $request)
	{
		$form = new HTMLForm(__CLASS__);
		
		$fieldset = new FormFieldsetHTML('blogs', $this->lang['blogs']);
		$form->add_fieldset($fieldset);
		
		$fieldset->add_field(new FormFieldTextEditor('name', $this->common_lang['form.name'], $this->get_blogs()->get_name(), array('required' => true)));

		if (!$this->is_contributor_member())
		{
			$fieldset->add_field(new FormFieldCheckbox('personalize_rewrited_name', $this->common_lang['form.rewrited_name.personalize'], $this->get_blogs()->rewrited_name_is_personalized(), array(
			'events' => array('click' => '
			if (HTMLForms.getField("personalize_rewrited_name").getValue()) {
				HTMLForms.getField("rewrited_name").enable();
			} else { 
				HTMLForms.getField("rewrited_name").disable();
			}'
			))));
			
			$fieldset->add_field(new FormFieldTextEditor('rewrited_name', $this->common_lang['form.rewrited_name'], $this->get_blogs()->get_rewrited_name(), array(
				'description' => $this->common_lang['form.rewrited_name.description'], 
				'hidden' => !$this->get_blogs()->rewrited_name_is_personalized()
			), array(new FormFieldConstraintRegex('`^[a-z0-9\-]+$`i'))));
		}
		
		$search_category_children_options = new SearchCategoryChildrensOptions();
		$search_category_children_options->add_authorizations_bits(Category::READ_AUTHORIZATIONS);
		$search_category_children_options->add_authorizations_bits(Category::CONTRIBUTION_AUTHORIZATIONS);
		$fieldset->add_field(BlogsService::get_categories_manager()->get_select_categories_form_field('id_cat', $this->common_lang['form.category'], $this->get_blogs()->get_id_cat(), $search_category_children_options));
		
		$fieldset->add_field(new FormFieldRichTextEditor('contents', $this->common_lang['form.contents'], $this->get_blogs()->get_contents(), array('rows' => 15, 'required' => true)));
		
		$fieldset->add_field(new FormFieldCheckbox('enable_short_contents', $this->lang['blogs.form.short_contents.enabled'], $this->get_blogs()->get_short_contents_enabled(), 
			array('description' => StringVars::replace_vars($this->lang['blogs.form.short_contents.enabled.description'], array('number' => BlogsConfig::load()->get_number_character_to_cut())), 'events' => array('click' => '
			if (HTMLForms.getField("enable_short_contents").getValue()) {
				HTMLForms.getField("short_contents").enable();
			} else { 
				HTMLForms.getField("short_contents").disable();
			}'))
		));
		
		$fieldset->add_field(new FormFieldRichTextEditor('short_contents', $this->lang['blogs.form.short_contents'], $this->get_blogs()->get_short_contents(), array(
			'hidden' => !$this->get_blogs()->get_short_contents_enabled(),
			'description' => !BlogsConfig::load()->get_display_condensed_enabled() ? '<span class="color-alert">' . $this->lang['blogs.form.short_contents.description'] . '</span>' : ''
		)));

		$other_fieldset = new FormFieldsetHTML('other', $this->common_lang['form.other']);
		$form->add_fieldset($other_fieldset);

		$other_fieldset->add_field(new FormFieldUploadFile('picture', $this->common_lang['form.picture'], $this->get_blogs()->get_picture()->relative()));

		$other_fieldset->add_field(BlogsService::get_keywords_manager()->get_form_field($this->get_blogs()->get_id(), 'keywords', $this->common_lang['form.keywords'], array('description' => $this->common_lang['form.keywords.description'])));
		
		$other_fieldset->add_field(new BlogsFormFieldSelectSources('sources', $this->common_lang['form.sources'], $this->get_blogs()->get_sources()));
		
		if (!$this->is_contributor_member())
		{
			$publication_fieldset = new FormFieldsetHTML('publication', $this->common_lang['form.approbation']);
			$form->add_fieldset($publication_fieldset);

			$publication_fieldset->add_field(new FormFieldDateTime('creation_date', $this->common_lang['form.date.creation'], $this->get_blogs()->get_creation_date(),
				array('required' => true)
			));
			
			$publication_fieldset->add_field(new FormFieldSimpleSelectChoice('approbation_type', $this->common_lang['form.approbation'], $this->get_blogs()->get_approbation_type(),
				array(
					new FormFieldSelectChoiceOption($this->common_lang['form.approbation.not'], Blogs::NOT_APPROVAL),
					new FormFieldSelectChoiceOption($this->common_lang['form.approbation.now'], Blogs::APPROVAL_NOW),
					new FormFieldSelectChoiceOption($this->common_lang['status.approved.date'], Blogs::APPROVAL_DATE),
				),
				array('events' => array('change' => '
				if (HTMLForms.getField("approbation_type").getValue() == 2) {
					jQuery("#' . __CLASS__ . '_start_date_field").show();
					HTMLForms.getField("end_date_enable").enable();
				} else { 
					jQuery("#' . __CLASS__ . '_start_date_field").hide();
					HTMLForms.getField("end_date_enable").disable();
				}'))
			));
			
			$publication_fieldset->add_field(new FormFieldDateTime('start_date', $this->common_lang['form.date.start'], ($this->get_blogs()->get_start_date() === null ? new Date() : $this->get_blogs()->get_start_date()), array('hidden' => ($this->get_blogs()->get_approbation_type() != Blogs::APPROVAL_DATE))));
			
			$publication_fieldset->add_field(new FormFieldCheckbox('end_date_enable', $this->common_lang['form.date.end.enable'], $this->get_blogs()->end_date_enabled(), array(
			'hidden' => ($this->get_blogs()->get_approbation_type() != Blogs::APPROVAL_DATE),
			'events' => array('click' => '
			if (HTMLForms.getField("end_date_enable").getValue()) {
				HTMLForms.getField("end_date").enable();
			} else { 
				HTMLForms.getField("end_date").disable();
			}'
			))));
			
			$publication_fieldset->add_field(new FormFieldDateTime('end_date', $this->common_lang['form.date.end'], ($this->get_blogs()->get_end_date() === null ? new Date() : $this->get_blogs()->get_end_date()), array('hidden' => !$this->get_blogs()->end_date_enabled())));
		
			$publication_fieldset->add_field(new FormFieldCheckbox('top_list', $this->lang['blogs.form.top_list'], $this->get_blogs()->top_list_enabled()));
		}
		
		$this->build_contribution_fieldset($form);
		
		$fieldset->add_field(new FormFieldHidden('referrer', $request->get_url_referrer()));
		
		$this->submit_button = new FormButtonDefaultSubmit();
		$form->add_button($this->submit_button);
		$form->add_button(new FormButtonReset());
		
		$this->form = $form;
	}
	
	private function build_contribution_fieldset($form)
	{
		if ($this->get_blogs()->get_id() === null && $this->is_contributor_member())
		{
			$fieldset = new FormFieldsetHTML('contribution', LangLoader::get_message('contribution', 'user-common'));
			$fieldset->set_description(MessageHelper::display($this->lang['blogs.form.contribution.explain'] . ' ' . LangLoader::get_message('contribution.explain', 'user-common'), MessageHelper::WARNING)->render());
			$form->add_fieldset($fieldset);
			
			$fieldset->add_field(new FormFieldRichTextEditor('contribution_description', LangLoader::get_message('contribution.description', 'user-common'), '', array('description' => LangLoader::get_message('contribution.description.explain', 'user-common'))));
		}
	}
	
	private function is_contributor_member()
	{
		return (!BlogsAuthorizationsService::check_authorizations()->write() && BlogsAuthorizationsService::check_authorizations()->contribution());
	}
	
	private function get_blogs()
	{
		if ($this->blogs === null)
		{
			$id = AppContext::get_request()->get_getint('id', 0);
			if (!empty($id))
			{
				try {
					$this->blogs = BlogsService::get_blogs('WHERE id=:id', array('id' => $id));
				} catch (RowNotFoundException $e) {
					$error_controller = PHPBoostErrors::unexisting_page();
					DispatchManager::redirect($error_controller);
				}
			}
			else
			{
				$this->is_new_blogs = true;
				$this->blogs = new Blogs();
				$this->blogs->init_default_properties(AppContext::get_request()->get_getint('id_category', Category::ROOT_CATEGORY));
			}
		}
		return $this->blogs;
	}
	
	private function check_authorizations()
	{
		$blogs = $this->get_blogs();
		
		if ($blogs->get_id() === null)
		{
			if (!$blogs->is_authorized_to_add())
			{
				$error_controller = PHPBoostErrors::user_not_authorized();
				DispatchManager::redirect($error_controller);
			}
		}
		else
		{
			if (!$blogs->is_authorized_to_edit())
			{
				$error_controller = PHPBoostErrors::user_not_authorized();
				DispatchManager::redirect($error_controller);
			}
		}
		if (AppContext::get_current_user()->is_readonly())
		{
			$controller = PHPBoostErrors::user_in_read_only();
			DispatchManager::redirect($controller);
		}
	}
	
	private function save()
	{
		$blogs = $this->get_blogs();
		
		$blogs->set_name($this->form->get_value('name'));
		$blogs->set_id_cat($this->form->get_value('id_cat')->get_raw_value());
		$blogs->set_contents($this->form->get_value('contents'));
		$blogs->set_short_contents(($this->form->get_value('enable_short_contents') ? $this->form->get_value('short_contents') : ''));
		$blogs->set_picture(new Url($this->form->get_value('picture')));
		
		$blogs->set_sources($this->form->get_value('sources'));
		
		if ($this->is_contributor_member())
		{
			if ($blogs->get_id() === null)
				$blogs->set_creation_date(new Date());
			
			$blogs->set_rewrited_name(Url::encode_rewrite($blogs->get_name()));
			$blogs->set_approbation_type(Blogs::NOT_APPROVAL);
			$blogs->clean_start_and_end_date();
		}
		else
		{
			$blogs->set_creation_date($this->form->get_value('creation_date'));
			$rewrited_name = $this->form->get_value('rewrited_name', '');
			$rewrited_name = $this->form->get_value('personalize_rewrited_name') && !empty($rewrited_name) ? $rewrited_name : Url::encode_rewrite($blogs->get_name());
			$blogs->set_rewrited_name($rewrited_name);
			$blogs->set_top_list_enabled($this->form->get_value('top_list'));
			$blogs->set_approbation_type($this->form->get_value('approbation_type')->get_raw_value());
			if ($blogs->get_approbation_type() == Blogs::APPROVAL_DATE)
			{
				$config = BlogsConfig::load();
				$deferred_operations = $config->get_deferred_operations();
				
				$old_start_date = $blogs->get_start_date();
				$start_date = $this->form->get_value('start_date');
				$blogs->set_start_date($start_date);
				
				if ($old_start_date !== null && $old_start_date->get_timestamp() != $start_date->get_timestamp() && in_array($old_start_date->get_timestamp(), $deferred_operations))
				{
					$key = array_search($old_start_date->get_timestamp(), $deferred_operations);
					unset($deferred_operations[$key]);
				}
				
				if (!in_array($start_date->get_timestamp(), $deferred_operations))
					$deferred_operations[] = $start_date->get_timestamp();
				
				if ($this->form->get_value('end_date_enable'))
				{
					$old_end_date = $blogs->get_end_date();
					$end_date = $this->form->get_value('end_date');
					$blogs->set_end_date($end_date);
					
					if ($old_end_date !== null && $old_end_date->get_timestamp() != $end_date->get_timestamp() && in_array($old_end_date->get_timestamp(), $deferred_operations))
					{
						$key = array_search($old_end_date->get_timestamp(), $deferred_operations);
						unset($deferred_operations[$key]);
					}
					
					if (!in_array($end_date->get_timestamp(), $deferred_operations))
						$deferred_operations[] = $end_date->get_timestamp();
				}
				else
				{
					$blogs->clean_end_date();
				}
				
				$config->set_deferred_operations($deferred_operations);
				BlogsConfig::save();
			}
			else
			{
				$blogs->clean_start_and_end_date();
			}
		}
		
		if ($blogs->get_id() === null)
		{
			$blogs->set_author_user(AppContext::get_current_user());
			$id_blogs = BlogsService::add($blogs);
		}
		else
		{
			$id_blogs = $blogs->get_id();
			BlogsService::update($blogs);
		}
		
		$this->contribution_actions($blogs, $id_blogs);
		
		BlogsService::get_keywords_manager()->put_relations($id_blogs, $this->form->get_value('keywords'));
		
		Feed::clear_cache('blogs');
	}
	
	private function contribution_actions(Blogs $blogs, $id_blogs)
	{
		if ($blogs->get_id() === null)
		{
			if ($this->is_contributor_member())
			{
				$contribution = new Contribution();
				$contribution->set_id_in_module($id_blogs);
				$contribution->set_description(stripslashes($this->form->get_value('contribution_description')));
				$contribution->set_entitled($blogs->get_name());
				$contribution->set_fixing_url(BlogsUrlBuilder::edit_blogs($id_blogs)->relative());
				$contribution->set_poster_id(AppContext::get_current_user()->get_id());
				$contribution->set_module('blogs');
				$contribution->set_auth(
					Authorizations::capture_and_shift_bit_auth(
						BlogsService::get_categories_manager()->get_heritated_authorizations($blogs->get_id_cat(), Category::MODERATION_AUTHORIZATIONS, Authorizations::AUTH_CHILD_PRIORITY),
						Category::MODERATION_AUTHORIZATIONS, Contribution::CONTRIBUTION_AUTH_BIT
					)
				);
				ContributionService::save_contribution($contribution);
			}
		}
		else
		{
			$corresponding_contributions = ContributionService::find_by_criteria('blogs', $id_blogs);
			if (count($corresponding_contributions) > 0)
			{
				$blogs_contribution = $corresponding_contributions[0];
				$blogs_contribution->set_status(Event::EVENT_STATUS_PROCESSED);

				ContributionService::save_contribution($blogs_contribution);
			}
		}
		$blogs->set_id($id_blogs);
	}
	
	private function redirect()
	{
		$blogs = $this->get_blogs();
		$category = $blogs->get_category();

		if ($this->is_new_blogs && $this->is_contributor_member() && !$blogs->is_visible())
		{
			DispatchManager::redirect(new UserContributionSuccessController());
		}
		elseif ($blogs->is_visible())
		{
			if ($this->is_new_blogs)
				AppContext::get_response()->redirect(BlogsUrlBuilder::display_blogs($category->get_id(), $category->get_rewrited_name(), $blogs->get_id(), $blogs->get_rewrited_name()), StringVars::replace_vars($this->lang['blogs.message.success.add'], array('name' => $blogs->get_name())));
			else
				AppContext::get_response()->redirect(($this->form->get_value('referrer') ? $this->form->get_value('referrer') : BlogsUrlBuilder::display_blogs($category->get_id(), $category->get_rewrited_name(), $blogs->get_id(), $blogs->get_rewrited_name())), StringVars::replace_vars($this->lang['blogs.message.success.edit'], array('name' => $blogs->get_name())));
		}
		else
		{
			if ($this->is_new_blogs)
				AppContext::get_response()->redirect(BlogsUrlBuilder::display_pending_blogs(), StringVars::replace_vars($this->lang['blogs.message.success.add'], array('name' => $blogs->get_name())));
			else
				AppContext::get_response()->redirect(($this->form->get_value('referrer') ? $this->form->get_value('referrer') : BlogsUrlBuilder::display_pending_blogs()), StringVars::replace_vars($this->lang['blogs.message.success.edit'], array('name' => $blogs->get_name())));
		}
	}
	
	private function generate_response(View $tpl)
	{
		$blogs = $this->get_blogs();
		
		$response = new SiteDisplayResponse($tpl);
		$graphical_environment = $response->get_graphical_environment();
		
		$breadcrumb = $graphical_environment->get_breadcrumb();
		$breadcrumb->add($this->lang['blogs'], BlogsUrlBuilder::home());
		
		if ($this->get_blogs()->get_id() === null)
		{
			$graphical_environment->set_page_title($this->lang['blogs.add'], $this->lang['blogs']);
			$breadcrumb->add($this->lang['blogs.add'], BlogsUrlBuilder::add_blogs($blogs->get_id_cat()));
			$graphical_environment->get_seo_meta_data()->set_description($this->lang['blogs.add']);
			$graphical_environment->get_seo_meta_data()->set_canonical_url(BlogsUrlBuilder::add_blogs($blogs->get_id_cat()));
		}
		else
		{
			$graphical_environment->set_page_title($this->lang['blogs.edit'], $this->lang['blogs']);
			$graphical_environment->get_seo_meta_data()->set_description($this->lang['blogs.edit']);
			$graphical_environment->get_seo_meta_data()->set_canonical_url(BlogsUrlBuilder::edit_blogs($blogs->get_id()));
			
			$categories = array_reverse(BlogsService::get_categories_manager()->get_parents($blogs->get_id_cat(), true));
			foreach ($categories as $id => $category)
			{
				if ($category->get_id() != Category::ROOT_CATEGORY)
					$breadcrumb->add($category->get_name(), BlogsUrlBuilder::display_category($category->get_id(), $category->get_rewrited_name()));
			}
			$category = $blogs->get_category();
			$breadcrumb->add($blogs->get_name(), BlogsUrlBuilder::display_blogs($category->get_id(), $category->get_rewrited_name(), $blogs->get_id(), $blogs->get_rewrited_name()));
			$breadcrumb->add($this->lang['blogs.edit'], BlogsUrlBuilder::edit_blogs($blogs->get_id()));
		}
		
		return $response;
	}
}
?>